%%% The code is written based on algorithm developed 
%%% by Lilia Maliar, Serguei Maliar and Fernando Valli to solve 
%%% incomplete market model with aggregate uncertainty and hetero agents 
%%% using Krusell-Smith Algorithm with Simulation. References are:
%%% MMV2010-JEDC: (and its working paper version)
%%%       Maliar, L., Maliar, S., & Valli, F. (2010). 
%%%       Solving the incomplete markets model with aggregate uncertainty 
%%%       using the Krusell–Smith algorithm. 
%%%       Journal of Economic Dynamics and Control, 34(1), 42-49.
%%% Den Haan2010-JEDC:
%%%       Den Haan, W. J. (2010). 
%%%       Comparison of solutions to the incomplete markets model with aggregate uncertainty. 
%%%       Journal of Economic Dynamics and Control, 34(1), 4-27.
%%% Website of Serguei Maliar:
%%%       https://web.stanford.edu/~maliars/Files/Codes.html

% The program includes the following files: 
%
% 1. "MAIN.m" (computes a solution and stores the results in "Solution")
% 2. "SHOCKS.m"      (a subroutine of MAIN.m; generates the shocks)
% 3. "INDIVIDUAL.m"  (a subroutine of MAIN.m; computes a solution to the 
%                     individual problem)
% 4. "AGGREGATE.m"   (a subroutine of MAIN.m; performs the stochastic 
%                        simulation)
% 5. "Inputs_for_test" (contains initial distribution of capital and 
%    10,000-period realizations of aggregate shock and idiosyncratic shock 
%    for one agent provided by Den Haan, Judd and Juillard, 2008) 
% 6. "TEST.m" (should be run after "MAIN.m"; it uses "Inputs_for_test" and
%    "Solution_to_model" for computing the statistics reported in Den Haan's
%    2008, comparison article)  

 
% __________________________________________________________________________
clc;
clear all;
%__________________________________________________________________________
%
% Parameters
%__________________________________________________________________________

beta=0.99;       % discount factor
gamma=1;         % utility-function parameter
alpha=0.36;      % share of capital in the production function
delta=0.025;     % depreciation rate
delta_a=0.01;    % (1-delta_a) is the productivity level in a bad state, 
                 % and (1+delta_a) is the productivity level in a good state
mu = 0.15;       % unemployment benefits as a share of wage
l_bar=1/0.9;     % time endowment; normalizes labor supply to 1 in a bad state
N=10000;         % number of agents for the stochastic simulation
T=1100;          % simulation length
ndiscard=100;    % number of periods to discard

nstates_id=2;    % number of states for the idiosyncratic shock (unemployed(u) or employed(e));
nstates_ag=2;    % number of states for the aggregate shock (good(g) or bad(b));

epsilon_u=0;     % idiosyncratic shock if the agent is unemployed(u)
epsilon_e=1;     % idiosyncratic shock if the agent is employed(e)

ur_b=0.1;        % unemployment rate in a bad aggregate state
er_b=(1-ur_b);   % employment rate in a bad aggregate state
ur_g=0.04;       % unemployment rate in a good aggregate state
er_g=(1-ur_g);   % employment rate in a good aggregate state

% Matrix of transition probabilities in Den Haan, Judd, Juillard (2008)
% s,e/s'e'   b,u         b,e        g,u          g,e
%  b,u      0.525       0.35       0.03125      0.09375 
%  b,e      0.038889    0.836111   0.002083     0.122917
%  g,u      0.09375     0.03125    0.291667     0.583333
%  g,e      0.009115    0.115885   0.024306     0.850694

% i.e., conditional on z=b, prob(z'=b|z=b)= 0.525+0.35=0.875; prob(z'=g|z=b)=0.125.
% i.e., conditional on e=u, prob(e'=u|e=u,z'=z=b)=0.525; prob(e'=u|e=u,z'=g,z=b)=0.03125.
% i.e., conditional on z=b,
% ur'=(0.9*0.038889+0.1*0.525)/prob(z'=b|z=b)=0.1;

prob=[0.525 0.35 0.03125 0.09375  
   0.038889 0.836111 0.002083 0.122917
   0.09375 0.03125 0.291667 0.583333
   0.009115 0.115885 0.024306 0.850694];

kss=((1/beta-(1-delta))/alpha)^(1/(alpha-1));
        % steady-state capital
        % in a deterministic model with employment rate of 0.9
        % (i.e., l_bar*L=1 where L is aggregate labor in the paper)

%__________________________________________________________________________
%
% Generation of shocks
%__________________________________________________________________________

% Stohcastic simulation uses both idiosyncartic and aggregate shocks

% call the subroutine SHOCK.m that generate aggregate and idiosyncartic shocks
[idshock,agshock]  = SHOCKS(prob,T,N,ur_b);
%__________________________________________________________________________
%
% Parameters of idiosyncratic and aggregate shocks
%__________________________________________________________________________

epsilon=zeros(nstates_id,1);  
% vector of possible idiosyncratic states VALUE
epsilon(1)=epsilon_u;   % epsilon_u=0
epsilon(2)=epsilon_e;   % epsilon_e=1
% the unemployed and employed states are 0 and 1, respectively

epsilon2=zeros(nstates_id,1); 
% vector of possible idiosyncratic states 
epsilon2(1)=1; 
epsilon2(2)=2; 
% the unemployed and employed states are 1 and 2, respectively  

a=zeros(nstates_ag,1);        
% vector of possible aggregate states VALUE
a(1)=1-delta_a; 
a(2)=1+delta_a; 
% bad and good agg. productivity are 1-delta_a and 1+delta_a, respectively

a2=zeros(nstates_ag,1);
% vector of possible aggregate states
a2(1)=1; 
a2(2)=2;             
% bad and good aggregate states are 1 and 2 respectively
%__________________________________________________________________________
%
% Grids
%__________________________________________________________________________

% Grid for capital in the individual problem

k_min=0;                   % minimum grid-value of capital
k_max=1000;                % maximum grid-value of capital
ngridk=100;                % number of grid points
x=linspace(0,0.5,ngridk)'; % generate a linear grid of ngridk points uniformly distributed on [0,0.5] interval
y=x.^7/max(x.^7);          % transform to polynomial distribution of grid points, formula (7) in the paper
k=k_min+(k_max-k_min)*y;   % transformation of grid points from [0,0.5] interval to [k_min,k_max] interval
% The grid is concentrated at the bottom (binding constraint), and is less concentrated at the bottom.

% Grid for mean of capital

km_min=30;                           % minimum grid-value of the mean of 
                                     % capital distribution, km 
km_max=50;                           % maximum grid value of km
ngridkm=4;                           % number of grid points for km 
km=linspace(km_min,km_max,ngridkm)'; % generate a grid of ngridkm points uniformly distributed on
                                     % [km_min,km_max] interval 
%__________________________________________________________________________
%
% Initial conditions: Step I out of III on pp.6 individual problem
%__________________________________________________________________________

kprime=zeros(ngridk,ngridkm,nstates_ag,nstates_id); 
     % next-period individual capital (k') depends on four state variables:
     % individual k, aggregate k, aggregate shock, idiosyncratic shock

% Guess initial capital evolution function 
for i=1:ngridkm
   for j=1:nstates_ag
      for h=1:nstates_id
         kprime(:,i,j,h)=0.9*k;       % k:grid points obtained from Grids
      end
   end
end

% Initial cross-sectional distribution of capital
                   % aggregate capital is chosen around steady state value, kss
    kcross=zeros(1,N)+kss;  % initial capital of all agents is equal to kss

% Initial vector of coefficients B of the ALM (in the paper, it is b)
                % the ALM in a bad state is ln(km')=B(1)+B(2)*ln(km)
                % the ALM in a good state is ln(km')=B(3)+B(4)*ln(km)

B=[0 1 0 1];
                % initial ALM in a bad state is set as ln(km')=0+ln(km)
                % initial ALM in a good state is set as ln(km')=0+ln(km)

%__________________________________________________________________________
%
% Convergence parameters: Step III out of III on pp.6 individual problem
%__________________________________________________________________________

dif_B=10^10;   % initial difference between coefficients B of the the ALM on
               % successive iterations; initially, set to a large number
criter_k=1e-8; % convergence criterion for the individual capital function
criter_B=1e-8; % convergence criterion for the coefficients B in the ALM
               % dif>criter: continue iteration; otherwise, stop
                                                     
update_k=0.7;  % updating parameter for the individual capital function
update_B=0.3;  % updating parameter for the coefficients B in the ALM
               % trade-off b/w existence and speed of convergence
%__________________________________________________________________________
%
% SOLVING THE MODEL (both Individual Problem and ALM)
%__________________________________________________________________________

iteration=0;      % initial iteration  
init_time=clock; % initialize the time clock

i=1;
while dif_B>criter_B
    % perform iterations until the difference between coefficients is less or equal than criter_B

    % call INDIVIDUAL subroutine
                 % solve for individual policy function that returns k' and c.
    [kprime,c]  = INDIVIDUAL(prob,ur_b,ur_g,ngridk,ngridkm,nstates_ag,nstates_id,k,km,er_b,er_g,a,epsilon,l_bar,alpha,delta,gamma,beta,mu,km_max,km_min,kprime,B,criter_k,k_min,k_max,update_k);

    % call AGGREGATE subroutine
                 % solve for mean of capital stock and cross-sectional distribution

    [kmts,kcross1]  = AGGREGATE(T,idshock,agshock,km_max,km_min,kprime,km,k,epsilon2,k_min,k_max,kcross,a2);

     % Time series for the ALM regression
     ibad=0;                % COUNT how many times the aggregate shock was bad
     igood=0;               % COUNT how many times the aggregate shock was good
     xbad=0;  ybad=0;       % regression-variables for a bad state
     xgood=0; ygood=0;      % regression-variables for a good state
   for i=ndiscard+1:T-1     % discard n period when doing the regression
      if agshock(i)==1      % bad state
      ibad=ibad+1;
      xbad(ibad,1)=log(kmts(i));   % log(K)
      ybad(ibad,1)=log(kmts(i+1)); % log(K')
      else                % agshock(i)==2, good state
      igood=igood+1;
      xgood(igood,1)=log(kmts(i));    % log(K)
      ygood(igood,1)=log(kmts(i+1));  % log(K')
      end
    end

  % regress Kt+1 on Kt to estimate ALM on different aggregate states: B1 obtained coefficients.
  [B1(1:2),s2,s3,s4,s5]=regress(ybad,[ones(ibad,1) xbad]);R2bad=s5(1);
    % run the OLS regression ln(km')=B(1)ones+B(2)*ln(km) for a bad agg. state
    % and compute R^2 (which is the first statistic in s5)
  [B1(3:4),s2,s3,s4,s5]=regress(ygood,[ones(igood,1) xgood]);R2good=s5(1);
    % make the OLS regression ln(km')=B(3)ones+B(4)*ln(km) for a good agg. state
    % and compute R^2 (which is the first statistic in s5)

   dif_B=norm(B-B1) % compute the difference between the initial and obtained
                 % vector of coefficients

    % To ensure that initial capital distribution comes from the ergodic set,
    % we use the terminal distribution of the current iteration as initial
    % distribution for a subsequent iteration. When the solution is sufficiently
    % accurate, dif_B<(criter_B*100), we stop such an updating and hold the
    % distribution "kcross" fixed for the rest of iterations. �

   if dif_B>(criter_B*100)
    kcross=kcross1; % the new capital distribution replaces the old one
   end

    B=B1*update_B+B*(1-update_B); % update the vector of the ALM coefficients
                 % according to the rule (9) in the paper
    iteration=iteration+1

end

end_time=clock;               % end the time clock
et=etime(end_time,init_time); % compute time in seconds that has elapsed 
                              % between init_time and end_time
disp('Elapsed Time (in seconds):'); et
disp('Iterations');          iteration
format long g; 
disp('R^2 bad aggregate shock:'); R2bad(1)
disp('R^2 good aggregare shock:'); R2good(1)
format; 
%__________________________________________________________________________
%
% FIGURE OF THE AGGREGATE TIME SERIES SOLUTION
%__________________________________________________________________________

kmalm=zeros(T,1);  % represents aggregate capital computed from the ALM
kmalm(1)=kmts(1);  % in the first period km computed from the ALM (kmalm) 
                   % is equal km computed from the cross-sectional capital 
                   % distribution (kmts)
                                   
for t=1:T-1       % compute kmalm for t=2:T
   if agshock(t)==1
      kmalm(t+1)=exp(B(1)+B(2)*log(kmalm(t)));
   else
      kmalm(t+1)=exp(B(3)+B(4)*log(kmalm(t)));
   end
   
end

Tts=1:1:T;
axis([min(Tts) max(Tts) min(kmts)*0.99 max(kmts)*1.01]); axis manual; hold on; 
plot (Tts,kmts(1:T,1),'-',Tts,kmalm(1:T,1),'--'),xlabel('Time'), ylabel('Aggregate capital series'), title('Figure 1. Accuracy of the aggregate law of motion.')
legend('implied by individual policy rule', 'aggregate law of motion')
%__________________________________________________________________________
%
% SAVE RESULTS IN FILE "Solution_to_model"
%__________________________________________________________________________
%
save Solution_to_model;
